'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('node:fs');
var node_buffer = require('node:buffer');
var process = require('node:process');
var unplugin$1 = require('unplugin');
var pluginutils = require('@rollup/pluginutils');

function check(code, options, length) {
    // check if fold is valid
    for (const [start, end] of options) {
        if (start > end) {
            return {
                valid: false,
                message: `Invalid input: start ${start} > end ${end}`,
            };
        }
    }
    // check if fold is in the code
    for (const [start, end] of options) {
        if (start > length || end > length) {
            return {
                valid: false,
                message: `Invalid input: start ${start} or end ${end} > length ${length}`,
            };
        }
    }
    return {
        valid: true,
        message: "",
    };
}

function processFold(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    options.forEach(([start, end]) => {
        // line start end
        // insert // ...
        const startIdx = code.findIndex((item) => item.lineNum === start);
        const endIdx = code.findIndex((item) => item.lineNum === end);
        code.splice(startIdx, endIdx - startIdx + 1, {
            lineNum: start,
            content: "// ...",
        });
    });
    return code;
}

function processTruncate(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    // sort by start
    options.sort((a, b) => a[0] - b[0]);
    let newCode = [];
    for (const [start, end] of options) {
        const startIdx = code.findIndex((item) => item.lineNum === start);
        const endIdx = code.findIndex((item) => item.lineNum === end);
        newCode = newCode.concat(code.slice(startIdx, endIdx + 1));
    }
    return newCode;
}

const defaultInclude = [
    "**/*.vue",
    "**/*.jsx",
    "**/*.tsx",
    "**/*.astro",
    "**/*.svelte",
    "**/*.html",
];
const defaultTagName = ["code-sample", "CodeSample", "codeSample"];
const unpluginFactory = (options = {}) => {
    return {
        name: "vite-plugin-code-sample",
        transformInclude(id) {
            return filter(options)(id);
        },
        transform(code, id) {
            return transform(code, id, options);
        },
    };
};
function filter(options) {
    return pluginutils.createFilter(options.include || defaultInclude, options.exclude);
}
function transform(code, id, options) {
    var _a;
    const tagName = (options === null || options === void 0 ? void 0 : options.tagName)
        ? options.tagName.concat(defaultTagName)
        : defaultTagName;
    const tagRegex = new RegExp(`<(${tagName.join("|")})([^>]*)>([\\s\\S]*?)<\\/\\1>`, "g");
    if (tagRegex.test(code)) {
        const removeInProd = (_a = options === null || options === void 0 ? void 0 : options.removeInProd) !== null && _a !== void 0 ? _a : true;
        if (removeInProd && process.env.NODE_ENV === "production") {
            // remove all code-sample tags in production
            return code.replace(tagRegex, "");
        }
        return code.replace(tagRegex, (match, tag, attributes, content) => {
            const fileSrcMatch = attributes.match(/source="([^"]+)"/);
            const fileContent = fileSrcMatch
                ? fs.readFileSync(fileSrcMatch[1], "utf-8")
                : fs.readFileSync(id, "utf-8");
            // 处理 fold 和 truncate 选项
            const foldOptRaw = attributes.match(/fold="([^"]+)"/);
            const truncateOptRaw = attributes.match(/truncate="([^"]+)"/);
            const foldOptions = foldOptRaw ? JSON.parse(foldOptRaw[1]) : [];
            const truncateOptions = truncateOptRaw
                ? JSON.parse(truncateOptRaw[1])
                : [];
            let metaLines = fileContent.split("\n").map((content, index) => ({
                lineNum: index + 1,
                content,
            }));
            const length = metaLines.length;
            // 处理 fold 和 truncate
            metaLines = processFold(metaLines, foldOptions, length);
            metaLines = processTruncate(metaLines, truncateOptions, length);
            const processedCode = metaLines.map((line) => line.content).join("\n");
            const base64Content = node_buffer.Buffer.from(processedCode).toString("base64");
            return `<${tag}${attributes} data-sample-code="${base64Content}">${content}</${tag}>`;
        });
    }
    return code;
}
const unplugin = /* #__PURE__ */ unplugin$1.createUnplugin(unpluginFactory);

exports.default = unplugin;
exports.filter = filter;
exports.transform = transform;
exports.unplugin = unplugin;
exports.unpluginFactory = unpluginFactory;
